from random import uniform

from testv.models.enums.input_type import InputTypeEnum
from testv.validators.exceptions import ValidationException
from testv.validators.rules.amount_ranges import T1AmountRanges, StrategyRanges, StrategyMinBidRanges, \
    StrategyGoalRanges, StrategyFrequencyRanges
from tests.unit.abstract_test_case import CommonTestCase


class AmountRangesTestCase(CommonTestCase):
    def setUp(self):
        self.random_percentage = round(uniform(StrategyRanges.PERCENTAGE_MIN_VALUE.value, StrategyRanges.PERCENTAGE_MAX_VALUE.value), 2)
        self.random_fixed = round(uniform(StrategyRanges.FIXED_MIN_VALUE.value, StrategyRanges.FIXED_MAX_VALUE.value), 2)

    def test_amount_can_be_negative_for_percentage(self):
        """
        Case: this is possible to provide negative value as amount for input_type percentage
        """
        try:
            T1AmountRanges(input_type=InputTypeEnum.PERCENTAGE,
                           amount=self.random_percentage,
                           type_ranges=StrategyRanges,
                           amount_field=self._faker.pystr()).execute()
        except Exception:
            self.fail("Unexpected exception")

    def test_amount_must_be_positive_for_non_percentage(self):
        """
        Case: value always should stay positive for input_type different from percentage
        """

        with self.assertRaises(ValidationException) as cm:
            T1AmountRanges(input_type=InputTypeEnum.FIXED,
                           amount=self.random_fixed * -1,
                           type_ranges=StrategyRanges,
                           amount_field="cap_amount").execute()

            self.assertRegex(str(cm.exception), "^valid range for cap_amount")

    def test_amount_should_be_in_valid_range(self):
        """
        Case: for input_type='percentage' range is -99.99 and 9999999.99
        for input_type='fixed' range is 0.01 and 9999999.99
        """

        with self.assertRaises(ValidationException) as cm:
            T1AmountRanges(input_type=InputTypeEnum.PERCENTAGE,
                           amount=StrategyRanges.PERCENTAGE_MIN_VALUE.value - 1,
                           type_ranges=StrategyRanges,
                           amount_field="cap_amount").execute()

            self.assertRegex(str(cm.exception), "(?i)^value for cap_amount when input_type=fixed must "
                                                "be between 0.01 and 9,999,999.99")

        with self.assertRaises(ValidationException) as cm:
            T1AmountRanges(input_type=InputTypeEnum.FIXED,
                           amount=StrategyRanges.FIXED_MAX_VALUE.value + 1,
                           type_ranges=StrategyRanges,
                           amount_field="cap_amount").execute()

            self.assertRegex(str(cm.exception), "(?i)^value for cap_amount")

    def test_strategy_amount_for_fixed_type_min_validation(self):
        """
        case: input_type='fixed' range should be: 0.01 - 9,999,999.99
        """

        with self.assertRaises(ValidationException) as cm:
            T1AmountRanges(input_type=InputTypeEnum.FIXED,
                           amount=StrategyRanges.FIXED_MIN_VALUE.value - 1,
                           type_ranges=StrategyRanges,
                           amount_field="cap_amount").execute()

        self.assertRegex(str(cm.exception), "(?i)^value for cap_amount")
        self.assertRegex(str(cm.exception), "(?i)between 0.01 and 9,999,999\.99$")

    def test_strategy_amount_for_fixed_type_max_validation(self):
        """
        case: input_type='fixed' range should be: 0 - 9,999,999.99
        """

        with self.assertRaises(ValidationException) as cm:
            T1AmountRanges(input_type=InputTypeEnum.FIXED,
                           amount=StrategyRanges.FIXED_MAX_VALUE.value + 1,
                           type_ranges=StrategyRanges,
                           amount_field="cap_amount").execute()

        self.assertRegex(str(cm.exception), "(?i)^value for cap_amount")
        self.assertRegex(str(cm.exception), "(?i)between 0.01 and 9,999,999\.99$")

    def test_strategy_amount_for_percentage_type_min_validation(self):
        """
        case:
        value for *_amount when input_type=percentage must be between -99.99 to 9,999,999.99
        """

        with self.assertRaises(ValidationException) as cm:
            T1AmountRanges(input_type=InputTypeEnum.PERCENTAGE,
                           amount=StrategyRanges.PERCENTAGE_MIN_VALUE.value - 1,
                           type_ranges=StrategyRanges,
                           amount_field="cap_amount").execute()

        self.assertRegex(str(cm.exception), "(?i)^value for cap_amount")
        self.assertRegex(str(cm.exception), "(?i)between -99.99 and 9,999,999\.99$")

    def test_strategy_amount_for_percentage_type_max_validation(self):
        """
        case:
        value for *_amount when input_type=percentage must be between -99.99 to 9,999,999.99
        """

        with self.assertRaises(ValidationException) as cm:
            T1AmountRanges(input_type=InputTypeEnum.PERCENTAGE,
                           amount=StrategyRanges.PERCENTAGE_MAX_VALUE.value + 1,
                           type_ranges=StrategyRanges,
                           amount_field="cap_amount").execute()

        self.assertRegex(str(cm.exception), "(?i)^value for cap_amount")
        self.assertRegex(str(cm.exception), "(?i)between -99.99 and 9,999,999\.99$")

    def test_bid_for_percentage_type_min_validation(self):
        """
        case:
        value for *_bid when input_type='percentage' must be between -99.99 to 9,999,999.99
        """

        with self.assertRaises(ValidationException) as cm:
            T1AmountRanges(input_type=InputTypeEnum.PERCENTAGE,
                           amount=StrategyMinBidRanges.PERCENTAGE_MIN_VALUE.value - 1,
                           type_ranges=StrategyMinBidRanges,
                           amount_field="bid_amount").execute()

        self.assertRegex(str(cm.exception), "(?i)^value for bid_amount")
        self.assertRegex(str(cm.exception), "(?i)between -99.99 and 9,999,999\.99$")

    def test_bid_for_percentage_type_max_validation(self):
        """
        case:
        value for *_bid when input_type='percentage' must be between -99.99 to 9,999,999.99
        """

        with self.assertRaises(ValidationException) as cm:
            T1AmountRanges(input_type=InputTypeEnum.PERCENTAGE,
                           amount=StrategyMinBidRanges.PERCENTAGE_MAX_VALUE.value + 1,
                           type_ranges=StrategyMinBidRanges,
                           amount_field="bid_amount").execute()

        self.assertRegex(str(cm.exception), "(?i)^value for bid_amount")
        self.assertRegex(str(cm.exception), "(?i)between -99.99 and 9,999,999\.99$")

    def test_goal_value_for_fixed_type_validation(self):
        """
        case:
        float value with decimals more than 3 in valid range should pass the validation
        """

        try:
            random_fixed = round(uniform(StrategyGoalRanges.FIXED_MIN_VALUE.value, StrategyGoalRanges.FIXED_MAX_VALUE.value), 5)
            T1AmountRanges(input_type=InputTypeEnum.FIXED,
                           amount=random_fixed,
                           type_ranges=StrategyGoalRanges,
                           amount_field="goal_value").execute()
        except BaseException as e:
            self.fail(str(e))

    def test_frequency_fixed_int_validation(self):
        """
        case:
        for frequency fixed type we have int values
        """

        with self.assertRaises(ValidationException) as cm:
            T1AmountRanges(input_type=InputTypeEnum.FIXED,
                           amount=StrategyFrequencyRanges.FIXED_MIN_VALUE.value - 1,
                           type_ranges=StrategyFrequencyRanges,
                           amount_field="frequency").execute()

        self.assertRegex(str(cm.exception), "(?i)between 1 and 9,999,999$")

    def test_frequency_percentage_float_validation(self):
        """
        case:
        for frequency percentage type we have float values
        """

        with self.assertRaises(ValidationException) as cm:
            T1AmountRanges(input_type=InputTypeEnum.PERCENTAGE,
                           amount=StrategyFrequencyRanges.PERCENTAGE_MIN_VALUE.value - 1,
                           type_ranges=StrategyFrequencyRanges,
                           amount_field="frequency").execute()

        self.assertRegex(str(cm.exception), "(?i)between -99.99 and 9,999,999.99$")
