from testv.models.enums.frequency_interval import FrequencyIntervalEnum
from testv.models.enums.frequency_type import FrequencyTypeEnum
from testv.models.enums.input_type import InputTypeEnum
from testv.validators.exceptions import ValidationException
from testv.validators.rules.frequency_type import FrequencyTypeRule
from tests.unit.abstract_test_case import CommonTestCase


class FrequencyTypeRuleTestCase(CommonTestCase):
    def test_execute_assert_failed(self):
        """
        case:
        frequency_type and frequency_interval should have valid type

        """

        with self.assertRaises(AssertionError):
            FrequencyTypeRule(frequency_optimization=None,
                              frequency_type=None,
                              frequency_amount=None,
                              frequency_interval=FrequencyIntervalEnum.set(None),
                              input_type=InputTypeEnum.set(None))

        with self.assertRaises(AssertionError):
            FrequencyTypeRule(frequency_optimization=None,
                              frequency_type=FrequencyTypeEnum.set(None),
                              frequency_amount=None,
                              frequency_interval=None,
                              input_type=InputTypeEnum.set(None))

        try:
            FrequencyTypeRule(frequency_optimization=None,
                              frequency_type=FrequencyTypeEnum.set(None),
                              frequency_amount=None,
                              frequency_interval=FrequencyIntervalEnum.set(None),
                              input_type=InputTypeEnum.set(None))
        except AssertionError:
            self.fail("Unexpected exception")

    def test_frequency_type_rules(self):
        """
            rules for 'frequency_type':
            required if frequency_optimization is FALSE
                - error message: "{:field_name} is required when frequency_optimization is FALSE"
            must be undefined if frequency_optimization is TRUE
                - error message: "{:field_name} cannot be defined when frequency_optimization is TRUE"
        """

        # required if frequency_optimization is FALSE
        with self.assertRaises(ValidationException):
            FrequencyTypeRule(frequency_optimization=False,
                              frequency_type=FrequencyTypeEnum.set(None),
                              frequency_amount=None,
                              frequency_interval=FrequencyIntervalEnum.set(None),
                              input_type=InputTypeEnum.set(None)).execute()

        # must be undefined if frequency_optimization is TRUE
        with self.assertRaises(ValidationException):
            FrequencyTypeRule(frequency_optimization=True,
                              frequency_type=FrequencyTypeEnum.NO_LIMIT,
                              frequency_amount=None,
                              frequency_interval=FrequencyIntervalEnum.set(None),
                              input_type=InputTypeEnum.set(None)).execute()

    def test_frequency_amount_rules(self):
        """
            rules for 'frequency_amount':
            required if frequency_type is asap or even
                - error message "{:field_name} is required when frequency_type is asap or even"
            must be undefined if frequency_optimization is TRUE
                - error message: "{:field_name} cannot be defined when frequency_optimization is TRUE"
            must be undefined if frequency_type is no-limit.
                - error message: "{:field_name} cannot be defined when frequency_type is no-limit"
        """

        # required if frequency_type is asap or even
        with self.assertRaises(ValidationException):
            FrequencyTypeRule(frequency_optimization=False,
                              frequency_type=FrequencyTypeEnum.EVEN,
                              frequency_amount=None,
                              frequency_interval=FrequencyIntervalEnum.set(None),
                              input_type=InputTypeEnum.set(None)).execute()

        # must be undefined if frequency_optimization is TRUE
        with self.assertRaises(ValidationException):
            FrequencyTypeRule(frequency_optimization=True,
                              frequency_type=FrequencyTypeEnum.NO_LIMIT,
                              frequency_amount=self._faker.pyfloat(),
                              frequency_interval=FrequencyIntervalEnum.set(None),
                              input_type=InputTypeEnum.set(None)).execute()

        # must be undefined if frequency_type is no-limit.
        with self.assertRaises(ValidationException):
            FrequencyTypeRule(frequency_optimization=False,
                              frequency_type=FrequencyTypeEnum.NO_LIMIT,
                              frequency_amount=self._faker.pyfloat(),
                              frequency_interval=FrequencyIntervalEnum.set(None),
                              input_type=InputTypeEnum.set(None)).execute()

    def test_frequency_interval_rules(self):
        """
            rules for 'frequency_interval':
            required if frequency_type is asap or even
                - error message "{:field_name} is required when frequency_type is asap or even"
            must be undefined if frequency_optimization is TRUE
                - error message: "{:field_name} cannot be defined when frequency_optimization is TRUE"
            must be undefined if frequency_type is no-limit.
                - error message: "{:field_name} cannot be defined when frequency_type is no-limit"
        """

        # required if frequency_type is asap or even
        with self.assertRaises(ValidationException):
            FrequencyTypeRule(frequency_optimization=False,
                              frequency_type=FrequencyTypeEnum.EVEN,
                              frequency_amount=self._faker.pyfloat(),
                              frequency_interval=FrequencyIntervalEnum.set(None),
                              input_type=InputTypeEnum.set(None)).execute()

        # must be undefined if frequency_optimization is TRUE
        with self.assertRaises(ValidationException):
            FrequencyTypeRule(frequency_optimization=True,
                              frequency_type=FrequencyTypeEnum.NO_LIMIT,
                              frequency_amount=None,
                              frequency_interval=FrequencyIntervalEnum.DAILY,
                              input_type=InputTypeEnum.set(None)).execute()

        # must be undefined if frequency_type is no-limit.
        with self.assertRaises(ValidationException):
            FrequencyTypeRule(frequency_optimization=False,
                              frequency_type=FrequencyTypeEnum.NO_LIMIT,
                              frequency_amount=None,
                              frequency_interval=FrequencyIntervalEnum.DAILY,
                              input_type=InputTypeEnum.set(None)).execute()

    def test_execute_valid(self):
        """
        case
        no exceptions for valid data
        :return:
        """

        try:
            FrequencyTypeRule(frequency_optimization=False,
                              frequency_type=FrequencyTypeEnum.NO_LIMIT,
                              frequency_amount=None,
                              frequency_interval=FrequencyIntervalEnum.UNDEFINED,
                              input_type=InputTypeEnum.set(None)).execute()
        except Exception:
            self.fail("Unexpected exception")

        try:
            FrequencyTypeRule(frequency_optimization=False,
                              frequency_type=FrequencyTypeEnum.EVEN,
                              frequency_amount=self._faker.pyfloat(),
                              frequency_interval=FrequencyIntervalEnum.DAILY,
                              input_type=InputTypeEnum.set(None)).execute()
        except Exception:
            self.fail("Unexpected exception")

        try:
            FrequencyTypeRule(frequency_optimization=True,
                              frequency_type=FrequencyTypeEnum.UNDEFINED,
                              frequency_amount=None,
                              frequency_interval=FrequencyIntervalEnum.UNDEFINED,
                              input_type=InputTypeEnum.set(None)).execute()
        except Exception:
            self.fail("Unexpected exception")

    def test_frequency_amount_int_for_fixed(self):
        """
        case:
        for frequency_amount, while input_type='fixed', type must be int
        """
        with self.assertRaises(ValidationException) as e:
            FrequencyTypeRule(frequency_optimization=False,
                              frequency_type=FrequencyTypeEnum.ASAP,
                              frequency_amount=self._faker.pyfloat(left_digits=3, right_digits=2),
                              frequency_interval=FrequencyIntervalEnum.DAILY,
                              input_type=InputTypeEnum.FIXED).execute()

        self.assertRegex(str(e.exception), "(?i)frequency_amount must be an integer$")

    def test_frequency_amount_float_for_percentage(self):
        """
        case:
        for frequency_amount, while input_type='percentage', type must be float
        """
        with self.assertRaises(ValidationException) as e:
            FrequencyTypeRule(frequency_optimization=False,
                              frequency_type=FrequencyTypeEnum.ASAP,
                              frequency_amount=self._faker.pystr(),
                              frequency_interval=FrequencyIntervalEnum.DAILY,
                              input_type=InputTypeEnum.PERCENTAGE).execute()

        self.assertRegex(str(e.exception), "(?i)frequency_amount must be numeric type")
