import random

from testv.models.enums.goal_type import GoalTypeEnum, PercentageGoalTypeEnum, FixedGoalTypeEnum
from testv.validators.exceptions import ValidationException
from testv.validators.rules.goal_type import GoalTypeRule
from tests.unit.abstract_test_case import CommonTestCase


class GoalTypeTestCase(CommonTestCase):
    def setUp(self):
        self.percentage_min_value = 1
        self.percentage_max_value = 100
        self.fixed_min_value = 1
        self.fixed_max_value = 9999999.9999

    def __getRandomGoalType(self) -> GoalTypeEnum:
        """
        Generate random goal type
        :return: GoalTypeEnum
        """
        return GoalTypeEnum.set(random.choice(PercentageGoalTypeEnum.asList() + FixedGoalTypeEnum.asList()))

    def __getRandomPercentageGoalType(self) -> GoalTypeEnum:
        """
        Generate random percentage goal type
        :return: GoalTypeEnum
        """
        return GoalTypeEnum.set(random.choice(PercentageGoalTypeEnum.asList()))

    def __getRandomFixedGoalType(self) -> GoalTypeEnum:
        """
        Generate random fixed goal type
        :return: GoalTypeEnum
        """
        return GoalTypeEnum.set(random.choice(FixedGoalTypeEnum.asList()))

    def test_execute_validation(self):
        """
        case:
        goal_value is validated according to goal_type
        """

        # goal_type should be one of ctr, vcr, viewability_rate, cpa, cpc, reach, roi, spend, vcpm
        with self.assertRaises(ValidationException):
            GoalTypeRule(goal_type=GoalTypeEnum.set(None),
                         goal_value=self._faker.pyint()).execute()

        # goal_value should be int or float
        with self.assertRaises(ValidationException):
            GoalTypeRule(goal_type=self.__getRandomGoalType(),
                         goal_value=self._faker.pystr()).execute()

        # goal_value should be 1-100 for percentage goal type
        with self.assertRaises(ValidationException):
            GoalTypeRule(goal_type=self.__getRandomPercentageGoalType(),
                         goal_value=self.percentage_min_value-1).execute()

        with self.assertRaises(ValidationException):
            GoalTypeRule(goal_type=self.__getRandomPercentageGoalType(),
                         goal_value=self.percentage_max_value+1).execute()

        # goal_value should be integer for percentage goal type
        with self.assertRaises(ValidationException):
            GoalTypeRule(goal_type=self.__getRandomPercentageGoalType(),
                         goal_value=self._faker.pyfloat(left_digits=2, right_digits=4, positive=True)).execute()

        # goal_value should be floated for fixed goal type
        with self.assertRaises(ValidationException):
            GoalTypeRule(goal_type=self.__getRandomFixedGoalType(),
                         goal_value=self._faker.pyint(min_value=self.percentage_min_value,
                                                      max_value=self.percentage_max_value)).execute()

        # goal_value should be 1-99999.9999 for fixed goal type
        with self.assertRaises(ValidationException):
            GoalTypeRule(goal_type=self.__getRandomFixedGoalType(),
                         goal_value=self.fixed_min_value-0.0001).execute()

        with self.assertRaises(ValidationException):
            GoalTypeRule(goal_type=self.__getRandomFixedGoalType(),
                         goal_value=self.fixed_max_value+0.0001).execute()

        # valid percentage goal type and value
        try:
            GoalTypeRule(goal_type=self.__getRandomPercentageGoalType(),
                         goal_value=self._faker.pyint(min_value=self.percentage_min_value,
                                                      max_value=self.percentage_max_value)).execute()
        except Exception:
            self.fail("Unexpected exception")

        # valid fixed goal type and value
        try:
            GoalTypeRule(goal_type=self.__getRandomFixedGoalType(),
                         goal_value=self._faker.pyfloat(left_digits=5, right_digits=4, positive=True)).execute()
        except Exception:
            self.fail("Unexpected exception")
