from testv.models.enums.impression_pacing_type import ImpressionPacingTypeEnum
from testv.models.enums.input_type import InputTypeEnum
from testv.models.enums.pacing_interval import PacingIntervalEnum
from testv.validators.exceptions import ValidationException
from testv.validators.rules.impression_pacing_type import ImpressionPacingTypeRule
from tests.unit.abstract_test_case import CommonTestCase


class ImpressionPacingTypeRuleTestCase(CommonTestCase):
    def test_execute_assert_failed(self):
        """
        case:
        impression_pacing_type and impression_pacing_interval should have valid type

        """
        with self.assertRaises(AssertionError):
            ImpressionPacingTypeRule(impression_pacing_type=None,
                                     impression_pacing_amount=None,
                                     impression_pacing_interval=None,
                                     input_type=InputTypeEnum.set(None))

        try:
            ImpressionPacingTypeRule(impression_pacing_type=ImpressionPacingTypeEnum.set(None),
                                     impression_pacing_amount=None,
                                     impression_pacing_interval=PacingIntervalEnum.set(None),
                                     input_type=InputTypeEnum.set(None))
        except AssertionError:
            self.fail("Unexpected exception")

    def test_pacing_amount_int_for_fixed(self):
        """
        case:
        for impression_pacing_amount, while input_type='fixed', type must be int
        """
        with self.assertRaises(ValidationException) as e:
            ImpressionPacingTypeRule(impression_pacing_type=ImpressionPacingTypeEnum.ASAP,
                                     impression_pacing_amount=self._faker.pyfloat(left_digits=2, right_digits=2),
                                     impression_pacing_interval=PacingIntervalEnum.DAY,
                                     input_type=InputTypeEnum.FIXED).execute()

        self.assertRegex(str(e.exception), "(?i)impression_pacing_amount must be an integer$")

    def test_pacing_amount_float_for_percentage(self):
        """
        case:
        for impression_pacing_amount, while input_type='percentage', type must be float
        """
        with self.assertRaises(ValidationException) as e:
            ImpressionPacingTypeRule(impression_pacing_type=ImpressionPacingTypeEnum.ASAP,
                                     impression_pacing_amount=self._faker.pystr(),
                                     impression_pacing_interval=PacingIntervalEnum.DAY,
                                     input_type=InputTypeEnum.PERCENTAGE).execute()

        self.assertRegex(str(e.exception), "(?i)impression_pacing_amount must be a float")

    def test_impression_pacing_type_is_defined(self):
        """
            case
            impression_pacing_amount required if impression_pacing_type is asap or even
                - error message "{:field_name} is required when impression_pacing_type is asap or even"
            must be undefined if impression_pacing_type is no-limit
                - error message "{:field_name} cannot be defined when impression_pacing_type is no-limit"
        """

        with self.assertRaises(ValidationException):
            ImpressionPacingTypeRule(impression_pacing_type=ImpressionPacingTypeEnum.ASAP,
                                     impression_pacing_amount=None,
                                     impression_pacing_interval=PacingIntervalEnum.set(None),
                                     input_type=InputTypeEnum.set(None)).execute()

        # must be undefined if impression_pacing_type is no-limit
        with self.assertRaises(ValidationException):
            ImpressionPacingTypeRule(impression_pacing_type=ImpressionPacingTypeEnum.NO_LIMIT,
                                     impression_pacing_amount=None,
                                     impression_pacing_interval=PacingIntervalEnum.DAY,
                                     input_type=InputTypeEnum.set(None)).execute()

    def test_impression_pacing_interval_is_defined(self):
        """
            case
            impression_pacing_interval required if impression_pacing_type is asap or even
                - error message "{:field_name} is required when impression_pacing_type is asap or even"
            must be undefined if impression_pacing_type is no-limit
                - error message "{:field_name} cannot be defined when impression_pacing_type is no-limit
        """

        with self.assertRaises(ValidationException):
            ImpressionPacingTypeRule(impression_pacing_type=ImpressionPacingTypeEnum.ASAP,
                                     impression_pacing_amount=None,
                                     impression_pacing_interval=PacingIntervalEnum.DAY,
                                     input_type=InputTypeEnum.set(None)).execute()

        with self.assertRaises(ValidationException):
            ImpressionPacingTypeRule(impression_pacing_type=ImpressionPacingTypeEnum.NO_LIMIT,
                                     impression_pacing_amount=self._faker.pyfloat(),
                                     impression_pacing_interval=PacingIntervalEnum.UNDEFINED,
                                     input_type=InputTypeEnum.set(None)).execute()

    def test_impression_type_is_required_when_interval_provided(self):
        """
        case
        impression_pacing_type is required if impression_pacing_interval is provided
        """
        with self.assertRaises(ValidationException) as e:
            ImpressionPacingTypeRule(impression_pacing_type=ImpressionPacingTypeEnum.UNDEFINED,
                                     impression_pacing_amount=None,
                                     impression_pacing_interval=PacingIntervalEnum.DAY,
                                     input_type=InputTypeEnum.set(None)).execute()

        self.assertRegex(str(e.exception), "(?i)^impression_pacing_type is required")

    def test_impression_interval_is_required_when_amount_provided(self):
        """
        case
        impression_pacing_interval is required if impression_pacing_amount is provided
        """
        with self.assertRaises(ValidationException) as e:
            ImpressionPacingTypeRule(impression_pacing_type=ImpressionPacingTypeEnum.UNDEFINED,
                                     impression_pacing_amount=self._faker.pyfloat(),
                                     impression_pacing_interval=PacingIntervalEnum.UNDEFINED,
                                     input_type=InputTypeEnum.set(None)).execute()

        self.assertRegex(str(e.exception), "(?i)^impression_pacing_interval is required")

    def test_execute_valid(self):
        """
        case
        no exceptions for valid data
        :return:
        """

        try:
            ImpressionPacingTypeRule(impression_pacing_type=ImpressionPacingTypeEnum.ASAP,
                                     impression_pacing_amount=self._faker.pyfloat(),
                                     impression_pacing_interval=PacingIntervalEnum.DAY,
                                     input_type=InputTypeEnum.set(None))
        except Exception:
            self.fail("Unexpected exception")

        try:
            ImpressionPacingTypeRule(impression_pacing_type=ImpressionPacingTypeEnum.NO_LIMIT,
                                     impression_pacing_amount=None,
                                     impression_pacing_interval=PacingIntervalEnum.UNDEFINED,
                                     input_type=InputTypeEnum.set(None))
        except Exception:
            self.fail("Unexpected exception")
