from random import choice

from testv.models.enums.impression_pacing_type import ImpressionPacingTypeEnum
from testv.models.enums.input_type import InputTypeEnum
from testv.models.enums.pacing_interval import PacingIntervalEnum
from testv.validators.rules.impression_pacing_type_put import ImpressionPacingTypePutRule
from tests.unit.abstract_test_case import CommonTestCase


class ImpressionPacingTypePutRuleTestCase(CommonTestCase):
    def test_pacing_type_assertion_failed(self):
        """
        case:
        impression_pacing_type should have a valid type

        """
        with self.assertRaises(AssertionError) as e:
            ImpressionPacingTypePutRule(impression_pacing_type=self._faker.pystr(),
                                        impression_pacing_amount=self._faker.pyfloat(),
                                        impression_pacing_interval=PacingIntervalEnum.UNDEFINED,
                                        input_type=InputTypeEnum.set(None))

        self.assertRegex(str(e.exception), "(?i)^impression_pacing_type")

    def test_pacing_interval_assertion_failed(self):
        """
        case:
        impression_pacing_interval should have a valid type

        """
        with self.assertRaises(AssertionError) as e:
            ImpressionPacingTypePutRule(impression_pacing_type=ImpressionPacingTypeEnum.UNDEFINED,
                                        impression_pacing_amount=self._faker.pyfloat(),
                                        impression_pacing_interval=self._faker.pyfloat(),
                                        input_type=InputTypeEnum.set(None))

        self.assertRegex(str(e.exception), "(?i)^impression_pacing_interval")

    def test_arguments_assertion_pass(self):
        """
        case:
        all arguments have valid types
        """
        try:
            ImpressionPacingTypePutRule(impression_pacing_type=ImpressionPacingTypeEnum.UNDEFINED,
                                        impression_pacing_amount=self._faker.pyfloat(),
                                        impression_pacing_interval=PacingIntervalEnum.UNDEFINED,
                                        input_type=InputTypeEnum.set(None))
        except Exception as e:
            self.fail(str(e))

    def test_pacing_type_no_limit_reset_fields(self):
        """
        case:
        while updating fields, passing pacing_type of no-limit should set rest of the fields:
        amount and interval
        """

        rule = ImpressionPacingTypePutRule(impression_pacing_type=ImpressionPacingTypeEnum.NO_LIMIT,
                                           impression_pacing_amount=self._faker.pyfloat(),
                                           impression_pacing_interval=PacingIntervalEnum.UNDEFINED,
                                           input_type=InputTypeEnum.set(None))

        expecting_amount = self._faker.pyfloat()
        expecting_interval = choice([PacingIntervalEnum.DAY, PacingIntervalEnum.HOUR])

        rule.put(pacing_amount=expecting_amount,
                 pacing_interval=expecting_interval)

        self.assertEqual(expecting_amount, rule.impression_pacing_amount)
        self.assertEqual(expecting_interval, rule.impression_pacing_interval)

    def test_pacing_type_even_keep_fields_untouched(self):
        """
        case:
        while updating fields, passing pacing_type of asap or even should not update rest of the fields:
        amount and interval
        """

        expecting_amount = self._faker.pyfloat()
        expecting_interval = PacingIntervalEnum.UNDEFINED

        rule = ImpressionPacingTypePutRule(impression_pacing_type=choice([ImpressionPacingTypeEnum.EVEN, ImpressionPacingTypeEnum.ASAP]),
                                           impression_pacing_amount=expecting_amount,
                                           impression_pacing_interval=expecting_interval,
                                           input_type=InputTypeEnum.set(None))

        rule.put(pacing_amount=self._faker.pyfloat(),
                 pacing_interval=choice([PacingIntervalEnum.DAY, PacingIntervalEnum.HOUR]))

        self.assertEqual(expecting_amount, rule.impression_pacing_amount)
        self.assertEqual(expecting_interval, rule.impression_pacing_interval)

    def test_put_reset_amount_to_int_for_fixed(self):
        """
        case: when input type switching to 'fixed', amount should be transformed into int
        """
        expecting_amount = self._faker.pyint()

        try:
            rule = ImpressionPacingTypePutRule(impression_pacing_type=choice([ImpressionPacingTypeEnum.EVEN, ImpressionPacingTypeEnum.ASAP]),
                                               impression_pacing_amount=expecting_amount,
                                               impression_pacing_interval=PacingIntervalEnum.DAY,
                                               input_type=InputTypeEnum.FIXED)

            item = rule.put(pacing_amount=expecting_amount, pacing_interval=PacingIntervalEnum.DAY)
            self.assertEqual(expecting_amount, item.impression_pacing_amount)
        except BaseException as e:
            self.fail(str(e))

    def test_put_reset_amount_to_int_should_work_with_none(self):
        """
        case: when input type switching to 'fixed', int transformation won't happen for None value
        """
        try:
            rule = ImpressionPacingTypePutRule(impression_pacing_type=choice([ImpressionPacingTypeEnum.EVEN, ImpressionPacingTypeEnum.ASAP]),
                                               impression_pacing_amount=None,
                                               impression_pacing_interval=PacingIntervalEnum.DAY,
                                               input_type=InputTypeEnum.FIXED)

            rule.put(pacing_amount=None, pacing_interval=PacingIntervalEnum.DAY)
        except BaseException as e:
            self.fail(str(e))
