import sys
import importlib as imp
import unittest
from faker import Faker
from random import choice
from mock import *

from testv.validators.rules.input_type_required_post import InputTypeRequiredPostRule
from testv.models.enums.input_type import InputTypeEnum


# noinspection PyTypeChecker
class InputTypeRequiredPostRuleTestCase(unittest.TestCase):
    """
    if t1_spend_cap_amount AND t1_impression_cap_amount are not defined in the request, input_type is not required.
    PID-2591
    """
    @classmethod
    def setUpClass(cls):
        cls._faker = Faker()

        module = Mock()
        inputTypeRuleMock = Mock(name="InputTypeRequiredRule", spec=["execute"])
        inputTypeRuleMock.execute.return_value = True

        module.InputTypeRequiredRule.return_value = inputTypeRuleMock

        from testv.validators.rules import input_type_required_post as m
        sys.modules["testv.validators.rules.input_type_required"] = module
        imp.reload(m)

        cls.inputTypeRuleMock = inputTypeRuleMock

    def setUp(self):
        pass

    def tearDown(self):
        self.inputTypeRuleMock.reset_mock()

    def test_wrong_type_cap_amount1(self):
        """
        case: cap_amount1 should be type of float or None
        """

        with self.assertRaises(AssertionError) as e:
            InputTypeRequiredPostRule(cap_amount1=self._faker.pystr(),
                                      cap_amount2=self._faker.pyfloat(),
                                      input_type=InputTypeEnum.UNDEFINED)

        self.assertRegex(str(e.exception), "^cap_amount1 should be")

    def test_wrong_type_cap_amount2(self):
        """
        case: cap_amount2 should be type of float or None
        """

        with self.assertRaises(AssertionError) as e:
            InputTypeRequiredPostRule(cap_amount1=self._faker.pyfloat(),
                                      cap_amount2=self._faker.pystr(),
                                      input_type=InputTypeEnum.UNDEFINED)

        self.assertRegex(str(e.exception), "^cap_amount2 should be")

    def test_wrong_type_input_type_failed(self):
        """
        case: input_type should be type of InputTypeEnum
        """

        with self.assertRaises(AssertionError) as e:
            InputTypeRequiredPostRule(cap_amount1=self._faker.pyfloat(),
                                      cap_amount2=self._faker.pyfloat(),
                                      input_type=None)

        self.assertRegex(str(e.exception), "^input_type should be")

    def test_arguments_are_valid(self):
        """
        case: all arguments are valid it should not trigger any exceptions
        """

        try:
            InputTypeRequiredPostRule(cap_amount1=self._faker.pyfloat(),
                                      cap_amount2=self._faker.pyfloat(),
                                      input_type=choice([InputTypeEnum.PERCENTAGE, InputTypeEnum.FIXED]))
        except BaseException as e:
            self.fail(str(e))

    def test_presence_of_cap_amount_cause_of_exception(self):
        """
        case: presence one of cap amounts should trigger InputTypeRequiredRule
        """

        try:
            InputTypeRequiredPostRule(cap_amount1=self._faker.pyfloat(),
                                      cap_amount2=self._faker.pyfloat(),
                                      input_type=InputTypeEnum.UNDEFINED).execute()

            self.inputTypeRuleMock.execute.assert_called_once()
        except BaseException as e:
            self.fail(str(e))

    def test_absence_of_cap_amount_skip_validation(self):
        """
        case: absence both cap amounts fields will ignore validation trigger
        """

        try:
            InputTypeRequiredPostRule(cap_amount1=None,
                                      cap_amount2=None,
                                      input_type=choice([InputTypeEnum.PERCENTAGE, InputTypeEnum.FIXED])).execute()

            self.inputTypeRuleMock.execute.assert_not_called()
        except BaseException as e:
            self.fail(str(e))
