import sys
import importlib as imp
import unittest
from faker import Faker
from mock import *

from testv.validators.rules.input_type_required_put import InputTypeRequiredPutRule
from testv.models.enums.input_type import InputTypeEnum


# noinspection PyTypeChecker
class InputTypeRequiredPutRuleTestCase(unittest.TestCase):
    """
    if t1_spend_cap_amount AND t1_impression_cap_amount are not defined in the request, input_type is not required.
    PID-2591
    """
    @classmethod
    def setUpClass(cls):
        cls._faker = Faker()

        module = Mock()
        inputTypeRuleMock = Mock(name="InputTypeRequiredRule", spec=["execute"])
        inputTypeRuleMock.execute.return_value = True

        module.InputTypeRequiredRule.return_value = inputTypeRuleMock

        from testv.validators.rules import input_type_required_put as m
        sys.modules["testv.validators.rules.input_type_required"] = module
        imp.reload(m)

        cls.inputTypeRuleMock = inputTypeRuleMock

    def setUp(self):
        pass

    def tearDown(self):
        self.inputTypeRuleMock.reset_mock()

    def test_wrong_type_pacing_amount(self):
        """
        case: pacing_amount should be type of float or None
        """

        with self.assertRaises(AssertionError) as e:
            InputTypeRequiredPutRule(pacing_amount=self._faker.pystr(),
                                     impression_pacing_amount=self._faker.pyfloat(),
                                     frequency_amount=self._faker.pyfloat(),
                                     min_bid=self._faker.pyfloat(),
                                     max_bid=self._faker.pyfloat(),
                                     goal_value=self._faker.pyfloat(),
                                     roi_target=self._faker.pyfloat(),
                                     input_type=InputTypeEnum.UNDEFINED)

        self.assertRegex(str(e.exception), "^pacing_amount should be")

    def test_wrong_type_min_bid(self):
        """
        case: min_bid should be type of float or None
        """

        with self.assertRaises(AssertionError) as e:
            InputTypeRequiredPutRule(pacing_amount=self._faker.pyfloat(),
                                     impression_pacing_amount=self._faker.pyfloat(),
                                     frequency_amount=self._faker.pyfloat(),
                                     min_bid=self._faker.pystr(),
                                     max_bid=self._faker.pyfloat(),
                                     goal_value=self._faker.pyfloat(),
                                     roi_target=self._faker.pyfloat(),
                                     input_type=InputTypeEnum.UNDEFINED)

        self.assertRegex(str(e.exception), "^min_bid should be")

    def test_wrong_type_max_bid(self):
        """
        case: max_bid should be type of float or None
        """

        with self.assertRaises(AssertionError) as e:
            InputTypeRequiredPutRule(pacing_amount=self._faker.pyfloat(),
                                     impression_pacing_amount=self._faker.pyfloat(),
                                     frequency_amount=self._faker.pyfloat(),
                                     min_bid=self._faker.pyfloat(),
                                     max_bid=self._faker.pystr(),
                                     goal_value=self._faker.pyfloat(),
                                     roi_target=self._faker.pyfloat(),
                                     input_type=InputTypeEnum.UNDEFINED)

        self.assertRegex(str(e.exception), "^max_bid should be")

    def test_wrong_type_goal_value(self):
        """
        case: goal_value should be type of float or None
        """

        with self.assertRaises(AssertionError) as e:
            InputTypeRequiredPutRule(pacing_amount=self._faker.pyfloat(),
                                     impression_pacing_amount=self._faker.pyfloat(),
                                     frequency_amount=self._faker.pyfloat(),
                                     min_bid=self._faker.pyfloat(),
                                     max_bid=self._faker.pyfloat(),
                                     goal_value=self._faker.pystr(),
                                     roi_target=self._faker.pystr(),
                                     input_type=InputTypeEnum.UNDEFINED)

        self.assertRegex(str(e.exception), "^goal_value should be")

    def test_wrong_type_input_type_failed(self):
        """
        case: input_type should be type of InputTypeEnum
        """

        with self.assertRaises(AssertionError) as e:
            InputTypeRequiredPutRule(pacing_amount=self._faker.pyfloat(),
                                     impression_pacing_amount=self._faker.pyfloat(),
                                     frequency_amount=self._faker.pyfloat(),
                                     min_bid=self._faker.pyfloat(),
                                     max_bid=self._faker.pyfloat(),
                                     goal_value=self._faker.pyfloat(),
                                     roi_target=self._faker.pyfloat(),
                                     input_type=None)

        self.assertRegex(str(e.exception), "^input_type should be")

    def test_arguments_are_valid(self):
        """
        case: all arguments are valid it should not trigger any exceptions
        """

        try:
            InputTypeRequiredPutRule(pacing_amount=self._faker.pyfloat(),
                                     impression_pacing_amount=self._faker.pyfloat(),
                                     frequency_amount=self._faker.pyfloat(),
                                     min_bid=self._faker.pyfloat(),
                                     max_bid=self._faker.pyfloat(),
                                     goal_value=self._faker.pyfloat(),
                                     roi_target=self._faker.pyfloat(),
                                     input_type=InputTypeEnum.UNDEFINED)
        except BaseException as e:
            self.fail(str(e))

    def test_presence_of_any_fields_cause_of_exception(self):
        """
        case: presence one of fields should trigger InputTypeRequiredRule
        """

        try:
            InputTypeRequiredPutRule(pacing_amount=self._faker.pyfloat(),
                                     impression_pacing_amount=self._faker.pyfloat(),
                                     frequency_amount=self._faker.pyfloat(),
                                     min_bid=self._faker.pyfloat(),
                                     max_bid=self._faker.pyfloat(),
                                     goal_value=self._faker.pyfloat(),
                                     roi_target=self._faker.pyfloat(),
                                     input_type=InputTypeEnum.UNDEFINED).execute()

            self.inputTypeRuleMock.execute.assert_called_once()
        except BaseException as e:
            self.fail(str(e))

    def test_absence_of_all_fields_skip_validation(self):
        """
        case: absence all fields will ignore validation trigger
        """

        try:
            InputTypeRequiredPutRule(pacing_amount=None,
                                     impression_pacing_amount=None,
                                     frequency_amount=None,
                                     min_bid=None,
                                     max_bid=None,
                                     goal_value=None,
                                     roi_target=None,
                                     input_type=InputTypeEnum.UNDEFINED)

            self.inputTypeRuleMock.execute.assert_not_called()
        except BaseException as e:
            self.fail(str(e))
