import random
from datetime import datetime, timedelta

from mock.mock import Mock
from parameterized import parameterized

from testv.models.enums.cap_type import CapTypeEnum
from testv.models.enums.frequency_interval import FrequencyIntervalEnum
from testv.models.enums.frequency_type import FrequencyTypeEnum
from testv.models.enums.goal_type import PercentageGoalTypeEnum, FixedGoalTypeEnum
from testv import T1Validator, T1Service
from testv.validators.campaign_post_validator import CampaignPostValidator
from tests.unit.models.campaign import CampaignDTO
from tests.unit.abstract_test_case import CommonTestCase

MANDATORY_MSG = "%s is a mandatory field"


class CampaignPostValidatorTestCase(CommonTestCase):
    goal_type_msg = "goal_type value must be one of the following values"
    date_param_format = "%Y-%m-%dT%H:%M"
    spend_cap_type_msg = "spend_cap_type must be one of ['even', 'asap', 'no-limit']"
    invalid_type_msg = "%s value must be a %s"

    def setUp(self):
        self.t1_creds = {"token": "token",
                         "host": "https://t1qa5.mediamath.com/api/v2.0"}

        t1_mock = Mock(spec=["getCurrencyRate", "isValidAdvertiser"])
        t1_mock.getCurrencyRate.return_value = 0.7
        t1_mock.isValidAdvertiser.return_value = True

        self.t1_service = t1_mock
        # self.t1_service = T1Service(self.t1_creds)
        # print(self.t1_service.isValidAdvertiser(106713))

        self.percentage_min_value = 1
        self.percentage_max_value = 100
        self.fixed_min_value = 1
        self.fixed_max_value = 9999999.9999
        self.frequency_min_value = 1
        self.frequency_max_value = 9999999
        self.dto = CampaignDTO(advertiser_id=106713,
                               campaign_name=self._faker.name(),
                               is_programmatic_guaranteed=False,
                               status=self._faker.pybool(),
                               spend_cap_automatic=False,
                               spend_cap_type=CapTypeEnum.EVEN.value,
                               spend_cap_amount=self._faker.pyfloat(left_digits=7, right_digits=2, positive=True),
                               zone_name=self._faker.timezone(),
                               start_date=(datetime.now() + timedelta(days=2)).strftime(self.date_param_format),
                               end_date=(datetime.now() + timedelta(days=3)).strftime(self.date_param_format),
                               currency_code=self._faker.currency_code(),
                               budget=self._faker.pyfloat(left_digits=7, right_digits=2, positive=True),
                               goal_type=random.choice([x.value for x in PercentageGoalTypeEnum]),
                               goal_value=self._faker.pyint(min_value=self.percentage_min_value,
                                                            max_value=self.percentage_max_value),
                               frequency_optimization=False,
                               frequency_type=random.choice([FrequencyTypeEnum.EVEN.value,
                                                             FrequencyTypeEnum.ASAP.value]),
                               frequency_amount=self._faker.pyint(min_value=self.frequency_min_value,
                                                                  max_value=self.frequency_max_value),
                               frequency_interval=random.choice([FrequencyIntervalEnum.HOUR.value,
                                                                 FrequencyIntervalEnum.DAILY.value,
                                                                 FrequencyIntervalEnum.MONTH.value,
                                                                 FrequencyIntervalEnum.WEEKLY.value]),
                               restrict_targeting_to_same_device_id=self._faker.pybool())
        self.dto.spend_cap_type = CapTypeEnum.EVEN.value

    def test_campaign_pass(self):
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, "")

    @parameterized.expand([
        ["advertiser_id", "integer"],
        ["campaign_name", "string"],
        ["is_programmatic_guaranteed", "boolean"],
        ["status", "boolean"],
        ["zone_name", "string"],
        ["start_date", "string"],
        ["end_date", "string"],
        ["currency_code", "string"],
        ["budget", "float"],
        ["goal_value", "float"],
        ["frequency_optimization", "boolean"],
        ["restrict_targeting_to_same_device_id", "boolean"]
    ])
    def test_campaign_missed_required_fields(self, param, value_type):
        self.dto.__setattr__(param, None)
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, value_type)

    def test_campaign_none_spend_cap_type(self):
        self.dto.spend_cap_type = None
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, "string")

    def test_campaign_none_goal_type(self):
        self.dto.goal_type = None
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, "string")

    def test_create_campaign_require_spend_cap_automatic_when_spend_cap_type_not_no_limit(self):
        """
        spend_cap_automatic is required when spend_cap_type is asap or even
        """
        self.dto.spend_cap_type = random.choice([CapTypeEnum.ASAP.value, CapTypeEnum.EVEN.value])
        self.dto.spend_cap_automatic = None
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, "spend_cap_automatic is required when spend_cap_type is asap or even")

    def test_create_campaign_not_require_spend_cap_automatic_when_spend_cap_type_is_no_limit(self):
        """
        spend_cap_automatic must be undefined when spend_cap_type is no-limit
        """
        self.dto.spend_cap_type = CapTypeEnum.NO_LIMIT.value
        self.dto.spend_cap_automatic = False
        self.dto.spend_cap_amount = None
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, "spend_cap_automatic must be undefined when spend_cap_type is no-limit")

    def test_create_campaign_not_require_spend_cap_amount_when_spend_cap_type_is_no_limit(self):
        """
        spend_cap_amount must be undefined when spend_cap_type is no-limit
        """
        self.dto.spend_cap_type = CapTypeEnum.NO_LIMIT.value
        self.dto.spend_cap_automatic = None
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, "spend_cap_amount must be undefined when spend_cap_type is no-limit")

    def test_create_campaign_not_require_spend_cap_amount_when_spend_cap_automatic_is_true(self):
        """
        spend_cap_amount must be undefined when spend_cap_type is no-limit
        """
        self.dto.spend_cap_automatic = True
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, "spend_cap_amount must be undefined when spend_cap_automatic is TRUE")

    def test_create_campaign_require_spend_cap_amount_when_spend_cap_automatic_is_false(self):
        """
        spend_cap_amount must be undefined when spend_cap_type is no-limit
        """
        self.dto.spend_cap_automatic = False
        self.dto.spend_cap_amount = None
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, "spend_cap_amount is required when spend_cap_automatic is FALSE")

    # test frequency validators
    def test_campaign_frequency_optimisation_true(self):
        self.dto.frequency_optimization = True
        self.dto.frequency_amount = None
        self.dto.frequency_type = FrequencyTypeEnum.UNDEFINED.value
        self.dto.frequency_interval = FrequencyIntervalEnum.UNDEFINED.value
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, "")

    def test_campaign_frequency_amount_should_not_be_set_when_frequency_optimization_true(self):
        self.dto.frequency_optimization = True
        self.dto.frequency_type = FrequencyTypeEnum.UNDEFINED.value
        self.dto.frequency_interval = FrequencyIntervalEnum.UNDEFINED.value
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, "frequency_amount cannot be defined when frequency_optimization is 'on'")

    def test_campaign_frequency_type_should_not_be_set_when_frequency_optimization_true(self):
        self.dto.frequency_optimization = True
        self.dto.frequency_amount = None
        self.dto.frequency_interval = FrequencyIntervalEnum.UNDEFINED.value
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, "frequency_type cannot be defined when frequency_optimization is 'on'")

    def test_campaign_frequency_interval_should_not_be_set_when_frequency_optimization_true(self):
        self.dto.frequency_optimization = True
        self.dto.frequency_amount = None
        self.dto.frequency_type = FrequencyTypeEnum.UNDEFINED.value
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator,
                                   "frequency_interval cannot be defined when frequency_optimization is 'on'")

    @parameterized.expand([
        [0],
        [10000000]
    ])
    def test_campaign_invalid_frequency_amount_value(self, value):
        self.dto.frequency_optimization = False
        self.dto.frequency_amount = value
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, "value for frequency_amount when input_type=fixed must be between 1 and "
                                              "9,999,999")

    def test_campaign_invalid_frequency_amount_type(self):
        self.dto.frequency_optimization = False
        self.dto.frequency_amount = 2.44
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, "for input_type='fixed' frequency_amount must be an integer")

    def test_create_campaign_require_frequency_type_when_frequency_optimization_false(self):
        """
        frequency_type is required when frequency_optimization is 'off'
        """
        self.dto.frequency_optimization = False
        self.dto.frequency_type = FrequencyTypeEnum.UNDEFINED.value
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, "frequency_type is required when frequency_optimization is 'off'")

    def test_create_campaign_not_require_frequency_type_when_frequency_optimization_true(self):
        """
        frequency_type must not be provided when frequency_optimization is true
        """
        self.dto.frequency_optimization = True
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, "frequency_type cannot be defined when frequency_optimization is 'on'")

    def test_create_campaign_require_frequency_interval_when_frequency_type_not_no_limit(self):
        """
        frequency_interval is required when frequency_type is asap or even
        """
        self.dto.frequency_optimization = False
        self.dto.frequency_interval = FrequencyIntervalEnum.UNDEFINED.value
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, "frequency_interval is required when frequency_type is asap or even")

    def test_create_campaign_require_frequency_amount_when_frequency_type_not_no_limit(self):
        """
        frequency_amount is required when frequency_type is asap or even
        """
        self.dto.frequency_optimization = False
        self.dto.frequency_amount = None
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, "frequency_amount is required when frequency_type is asap or even")

    def test_create_campaign_not_require_frequency_amount_when_frequency_type_is_no_limit(self):
        """
        frequency_amount must not be provided when frequency_type is no-limit
        """
        self.dto.frequency_optimization = False
        self.dto.frequency_type = FrequencyTypeEnum.NO_LIMIT.value
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, "frequency_amount cannot be defined when frequency_type is no-limit")

    def test_create_campaign_not_require_frequency_interval_when_frequency_type_is_no_limit(self):
        """
        frequency_interval must not be provided when frequency_type is no-limit
        """
        self.dto.frequency_optimization = False
        self.dto.frequency_type = FrequencyTypeEnum.NO_LIMIT.value
        self.dto.frequency_amount = None
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, "frequency_interval cannot be defined when frequency_type is no-limit")

    def test_create_campaign_advertiser_id_0(self):
        """
        advertiser id must be positive number greater then 0
        """
        self.dto.advertiser_id = 0
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, "1")

    def test_create_campaign_advertiser_id_string(self):
        """
        advertiser id must be positive number greater then 0
        """
        self.dto.advertiser_id = self._faker.name()
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, "integer")

    def test_create_campaign_campaign_name_not_string(self):
        """
        campaign_name must a string from 2 to 256 characters long
        """
        self.dto.campaign_name = 0
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, "string")

    def test_create_campaign_campaign_name_invalid_length(self):
        """
        campaign_name must a string from 2 to 256 characters long
        """
        self.dto.campaign_name = "a"
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, "campaign_name is too short")

    @parameterized.expand([
        ["restrict_targeting_to_same_device_id"],
        ["frequency_optimization"],
        ["status"]
    ])
    def test_create_campaign_invalid_bool_values(self, param):
        """
        param must be an integer 0 or 1
        """
        self.dto.__setattr__(param, 2)
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, "boolean")

    def test_create_campaign_invalid_spend_cap_automatic_value(self):
        """
        spend_cap_automatic must be an integer 0 or 1
        """
        self.dto.spend_cap_automatic = 2
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, "boolean")

    @parameterized.expand([
        ["status", "boolean"],
        ["restrict_targeting_to_same_device_id", "boolean"],
        ["frequency_optimization", "boolean"],
    ])
    def test_create_campaign_status_invalid_value_type(self, param, value_type):
        """
        param must be an integer 0 or 1
        """
        self.dto.__setattr__(param, self._faker.name())
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, value_type)

    def test_create_campaign_status_spend_cap_automatic_value_type(self):
        """
        spend_cap_automatic must be an integer 0 or 1
        """
        self.dto.spend_cap_automatic = self._faker.name()
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, "boolean")

    @parameterized.expand([
        ["budget_min", "budget", 0],
        ["budget_max", "budget", 10000000]
    ])
    def test_campaign_invalid_budget_value(self, name, param, value):
        self.dto.__setattr__(param, value)
        self.dto.currency_code = "USD"
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, f"budget is out of range")

    def test_campaign_valid_not_uds_budget_value(self):
        self.dto.budget = 10000000
        self.dto.currency_code = "BYN"
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, "")

    @parameterized.expand([
        ["cap_amount_min", "spend_cap_amount", 0, 1, 9999999.99],
        ["cap_amount_max", "spend_cap_amount", 10000000, 1, 9999999.99]
    ])
    def test_campaign_invalid_cap_amount_value(self, name, param, value, min_value, max_value):
        self.dto.__setattr__(param, value)
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, f"value for {param} when input_type=fixed must be"
                                              f" between {min_value:,.2f} and {max_value:,.2f}")

    @parameterized.expand([
        ["frequency_amount_min", "frequency_amount", 0, 1, 9999999],
        ["frequency_amount_max", "frequency_amount", 10000000, 1, 9999999]
    ])
    def test_campaign_invalid_budget_value(self, name, param, value, min_value, max_value):
        self.dto.__setattr__(param, value)
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, f"value for {param} when input_type=fixed must be between {min_value} "
                                              f"and {max_value:,.0f}")

    def test_campaign_invalid_numeric_invalid_budget_value_type(self):
        self.dto.budget = self._faker.name()
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, "float")

    @parameterized.expand([
        ["spend_cap_amount"],
        ["frequency_amount"]
    ])
    def test_campaign_invalid_numeric_invalid_amounts_value_type(self, param):
        self.dto.__setattr__(param, self._faker.name())
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, f"{param} must be numeric type")

    def test_campaign_start_date_before_end_date(self):
        """
        start date must be before end_date
        """
        self.dto.start_date = "2020-01-03T12:00"
        self.dto.end_date = "2020-01-01T15:00"
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, f"start_date must be earlier than end_date")

    def test_campaign_start_date_end_date_not_the_same(self):
        """
        start_time must be earlier than end_time
        """
        self.dto.start_date = "2020-01-03T12:00"
        self.dto.end_date = "2020-01-03T12:00"
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, f"start_time must be earlier than end_time")

    def test_campaign_start_date_in_the_past(self):
        self.dto.start_date = "2020-01-03T12:00"
        self.dto.end_date = "2020-01-03T12:03"
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, f"start_date must be in future")

    def test_campaign_pg(self):
        self.dto.is_programmatic_guaranteed = True
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, f"spend_cap_type must be 'no-limit' when campaign type is PG")

    def test_campaign_goal_type_invalid(self):
        name = self._faker.name()
        self.dto.goal_type = name
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, self.goal_type_msg)

    def test_campaign_goal_value_percentage(self):
        self.dto.goal_type = random.choice([x.value for x in PercentageGoalTypeEnum])
        self.dto.goal_value = 0
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, f"value for goal_value when input_type=percentage "
                                              f"must be between 1 and 100")

    def test_campaign_goal_value_fixed(self):
        self.dto.goal_type = random.choice([x.value for x in FixedGoalTypeEnum])
        self.dto.goal_value = self.fixed_min_value-0.0001
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, f"value for goal_value when input_type=fixed must be between "
                                              f"1.0000 and 9,999,999.9999")

    def test_campaign_goal_value_invalid_value_type(self):
        self.dto.goal_value = self._faker.name()
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, "float")

    def test_campaign_invalid_zone_name(self):
        self.dto.zone_name = self._faker.pystr()
        validator = CampaignPostValidator(dto=self.dto.__dict__, t1_service=self.t1_service)
        self.check_validator_error(validator, "zone_name must be one of correct timezone names, "
                                              "e.g. 'America/New_York'")
